#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT

from hatchling.plugin import hookimpl

from .plugin import PysideBuildHook


@hookimpl
def hatch_register_build_hook():
    return PysideBuildHook


#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import os
import pathlib
import shutil
import tempfile

try:
    # noinspection PyUnresolvedReferences
    import tomllib
    patch_str = 'tomllib.load'
except ImportError:
    # noinspection PyUnresolvedReferences
    import tomli as tomllib
    patch_str = 'tomli.load'
import unittest
from unittest.mock import patch

from hatch_pyside import config


class ConfigMockTest(unittest.TestCase):
    @patch('builtins.open')
    def test_no_project(self, op):
        """If pyproject.toml is not found: OSError"""
        op.side_effect = OSError
        with self.assertRaises(OSError):
            config.get_project_folders()

    @patch(patch_str)
    @patch('builtins.open')
    def test_no_config(self, _op, load):
        load.return_value = {'project': {'name': 'foo'}}
        _, lst = config.get_project_folders()
        self.assertIsNone(lst)

    @patch(patch_str)
    @patch('builtins.open')
    def test_hatch_global(self, _op, load):
        lst = ['src/foo']
        load.side_effect = [
            {'project': {'name': 'foo'}},
            {'build': {'hooks': {'pyside': {'folders': lst}}},
                     'project': {'name': 'foo'}}]
        self.assertIs(lst, config.get_project_folders()[1])

    @patch(patch_str)
    @patch('builtins.open')
    def test_hatch_wheel(self, _op, load):
        lst = ['src/foo']
        load.side_effect = [
            {'project': {'name': 'foo'}},
            {'build':
                 {'targets': {'wheel': {'hooks':{'pyside': {'folders': lst}}}}},
             'project': {'name': 'foo'}}
        ]
        self.assertIs(lst, config.get_project_folders()[1])


class ConfigTempTest(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.TemporaryDirectory()
        self.old_dir = os.getcwd()
        os.chdir(self.tempdir.name)
        self.data_path = pathlib.Path(__file__).parent / 'data'

    def tearDown(self):
        os.chdir(self.old_dir)
        self.tempdir.cleanup()

    def test_hatch(self):
        shutil.copy(self.data_path / 'pyproject.empty', 'pyproject.toml')
        shutil.copy(self.data_path / 'hatch.wheel', 'hatch.toml')
        self.assertEqual([ 'src/bar'], config.get_project_folders()[1])

    def test_pyproj(self):
        shutil.copy(self.data_path / 'pyproject.toml', 'pyproject.toml')
        self.assertEqual(['src/foo'], config.get_project_folders()[1])

    def test_no_proj(self):
        with self.assertRaises(OSError):
            config.get_project_folders()

    def test_no_config(self):
        shutil.copy(self.data_path / 'pyproject.empty', 'pyproject.toml')
        self.assertIsNone(config.get_project_folders()[1])

if __name__ == '__main__':
    unittest.main()



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
"""The main GUI file"""
from typing import Optional

from PySide6.QtWidgets import QApplication, QMessageBox, QDialog

from hatch_pyside import config
from hatch_pyside.gui.mainwindow import MainWindow
from hatch_pyside.gui.ui_project_chooser_dlg import Ui_ProjectChooser


def main():
    """Starts the Qt application"""
    app = QApplication()
    try:
        name, projects = config.get_project_folders()
    except OSError:
        QMessageBox(
            QMessageBox.Icon.Critical,
            "Fatal",
            "No pyproject.toml file found",
            informativeText="The current working directory must "
            "contain a pyproject.toml file",
        ).exec()
        return 2
    except KeyError:
        QMessageBox(
            QMessageBox.Icon.Critical,
            "Fatal",
            "pyproject.toml has no project name",
            informativeText="The pyproject.toml file must "
            "at least contain [project] name=...",
        ).exec()
        return 2
    if projects is None or len(projects) == 0:
        QMessageBox(
            QMessageBox.Icon.Critical,
            "Fatal",
            "No hatch-pyside configuration found",
            informativeText="hatch-pyside plugin must be configured in "
            "pyproject.toml or hatch.toml",
        ).exec()
        return 1

    folder = choose_project(projects)
    if folder is None:  # User pressed the cancel button
        return 0

    main_wnd = MainWindow(projects, folder, name)
    main_wnd.show()

    return app.exec()


def choose_project(projects: list[str]) -> Optional[str]:
    """
    Returns a Qt project folder.

    projects should not be empty. If it contains more than one file,
    displays a QDialog to allow the user to choose one of them.

    Args:
        projects: the list of the configured folders for the plugin

    Returns:
        the project if there is only one, or the chosen project
    """
    if len(projects) == 1:
        return projects[0]
    dlg = DialogChooser(projects)
    if dlg.exec() == 0:
        return None
    return dlg.projectList.selectedItems()[0].text()


class DialogChooser(QDialog, Ui_ProjectChooser):
    """A QDialog to choose a project folder from"""

    def __init__(self, projects):
        super().__init__()
        self.setupUi(self)
        self.projectList.addItems(projects)
        self.projectList.setCurrentRow(0)


# not really useful here, but required by pyside6-project
if __name__ == "__main__":
    main()



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import os
from subprocess import CalledProcessError
from typing import Any

from hatchling.bridge.app import Application
from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from ..build import build, clean

class PysideBuildHook(BuildHookInterface):
    PLUGIN_NAME = 'pyside'

    def clean(self, _versions: list[str]) -> None:
        for f in self.config['folders']:
            try:
                self.app.display_debug(f'hatch-pyside build {f}', 1)
                cp = clean(os.path.join(self.root, f))
                self.app.display_debug(cp.stdout, 2)
            except CalledProcessError as e:
                dump(e, self.app)

    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        for f in self.config['folders']:
            try:
                self.app.display_debug(f'hatch-pyside build {f}', 1)
                cp = build(os.path.join(self.root, f))
                self.app.display_debug(cp.stdout, 2)
            except CalledProcessError as e:
                dump(e, self.app)
                self.app.abort('Fatal', e.returncode)
        build_data['artifacts'].append('ui_*.py')


def dump(e: CalledProcessError, app: Application):
    app.display_error(f'Command {e.cmd} returned {e.returncode}\n'
                      f'{e.stdout}\n' f'{e.stderr}\n')



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
"""Tiny wrapper to allow the package to be used as a module"""

import sys

from hatch_pyside.gui import app

if __name__ == "__main__":
    sys.exit(app.main())



# SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#
# SPDX-License-Identifier: MIT
"""Main package: declares version, clean and build"""

from .__about__ import __version__
from .build import *



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import os
import pathlib
import shutil
import subprocess
import tempfile
import unittest


class BuildTest(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.TemporaryDirectory()
        self.old_dir = os.getcwd()
        os.chdir(self.tempdir.name)
        self.data_path = pathlib.Path(__file__).parent / 'data'

    def tearDown(self):
        os.chdir(self.old_dir)
        self.tempdir.cleanup()

    def test_build(self):
        shutil.copy(self.data_path / 'pyproject.toml', '.')
        shutil.copytree(self.data_path / 'src', 'src')
        os.mkdir('dist')
        cr = subprocess.run(['pip', 'wheel', '-w', 'dist', self.data_path.parent.parent],
                            capture_output=True)
        self.assertEqual(0, cr.returncode)
        cr = subprocess.run(['pip', 'install', '.', '-f', 'dist'])
        self.assertEqual(0, cr.returncode)
        try:
            import foo.ui_MainWindow
        finally:
            subprocess.run(['pip', 'uninstall', '-y', 'foo'])


if __name__ == "__main__":
    unittest.main()



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
import os
import sys
from subprocess import CalledProcessError
from typing import Any

from hatchling.builders.hooks.plugin.interface import BuildHookInterface

sys.path.append(os.path.dirname(__file__))
from hatch_pyside.build import build, clean
from hatch_pyside.plugin.plugin import dump


# A custom plugin to build the GUI
class CustomBuildHook(BuildHookInterface):

    def clean(self, _versions: list[str]) -> None:
        try:
            clean(os.path.join(self.root, 'src', 'hatch_pyside', 'gui'))
        except CalledProcessError as e:
            dump(e, self.app)

    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        try:
            build(os.path.join(self.root, 'src', 'hatch_pyside', 'gui'))
        except CalledProcessError as e:
            dump(e, self.app)
            self.app.abort('Fatal', e.returncode)



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
"""Manages the extraction of relevant parameters from a pyproject.toml file"""
from typing import Any, Optional

# a hack to cope with tomllib not present in Python 3.9
try:
    # noinspection PyUnresolvedReferences
    import tomllib
except ImportError:
    # noinspection PyUnresolvedReferences
    import tomli as tomllib


def get_values(cfg: dict[str, Any]) -> Optional[list[str]]:
    """
    Extract the cumulative config for the pyside plugin: both global and wheel
    Args:
        cfg: the content of the hatch.toml file or of
        the tool.hatch table of pyproject.toml

    Returns:
        the relevant config

    Raises:
        KeyError    if no config is present for the plugin
    """
    try:
        return cfg["build"]["targets"]["wheel"]["hooks"]["pyside"]["folders"]
    except KeyError:
        return cfg["build"]["hooks"]["pyside"]["folders"]


def get_project_folders() -> tuple[str, Optional[list[str]]]:
    """
    Extract hatch-pyside config from pyproject.toml or hatch.toml

    Returns:
        the relevant config or None

    Raises:
        KeyError    if project.name is not found in pyproject.toml
        OSError     if pyproject.toml is not found
    """
    name = None
    try:
        with open("pyproject.toml", "rb") as fd:
            json = tomllib.load(fd)
            name = json["project"]["name"]
            return name, get_values(json["tool"]["hatch"])
    except KeyError:
        if name is None:
            raise
        try:
            with open("hatch.toml", "rb") as fd:
                return name, get_values(tomllib.load(fd))
        except (OSError, KeyError):
            return name, None



#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
"""This module wraps the calls to the pyside6-project command
used to build the relevant .py file from the Qt source ones
or to remove them.

command contains the full path of the first found pyside-project command.
"""
import shutil
import subprocess
import sysconfig

__all__ = ["command", "build", "clean"]

command: list[str]


try:
    _creation_flags = subprocess.CREATE_NO_WINDOW
except AttributeError:
    _creation_flags = 0

def run(cmd, folder) -> subprocess.CompletedProcess:
    """
    Pass the subcommand cmd to pyside6-project with the folder parameter
    Args:
        cmd:    the subcommand to execute (should be clean, lupdate or build)
        folder: the folder containing the .pyproject file

    Raises:
        CalledProcessError  if the pyside6-project command returns a non 0 exit code
    """
    return subprocess.run(
        command + [cmd, folder],
        stdin=subprocess.DEVNULL,
        capture_output=True,
        text=True,
        check=True,
        creationflags=_creation_flags,
    )


def build(folder: str) -> subprocess.CompletedProcess:
    """
    Calls run to build the dependant files
    Args:
        folder:  the folder containing the .pyproject file

    Raises:
        CalledProcessError  if the pyside6-project command returns a non 0 exit code
    """
    return run("build", folder)


def clean(folder: str) -> subprocess.CompletedProcess:
    """
    Calls run to clean the dependant files
    Args:
        folder:  the folder containing the .pyproject file

    Raises:
        CalledProcessError  if the pyside6-project command returns a non 0 exit code
    """
    return run("clean", folder)


def lupdate(folder: str) -> subprocess.CompletedProcess:
    """
    Calls run to update the translation files
    Args:
        folder:  the folder containing the .pyproject file

    Raises:
        CalledProcessError  if the pyside6-project command returns a non 0 exit code
    """
    return run("lupdate", folder)


def _build_command() -> str:
    # first search the command in the path
    prog = "pyside6-project"
    cmd = shutil.which(prog)
    if cmd is None:
        # else search it in any possible script installation folder
        visited = set()
        for d in (
            sysconfig.get_path("scripts", scheme)
            for scheme in sysconfig.get_scheme_names()
        ):
            if d in visited:  # only search once in each folder
                continue
            visited.add(d)
            cmd = shutil.which(prog)
            if cmd is not None:
                break
        else:
            raise RuntimeError(f"No installation of {prog} found")
    return cmd


# compute command at import time
command = [_build_command()]




#  SPDX-FileCopyrightText: 2025-present s-ball <s-ball@laposte.net>
#  #
#  SPDX-License-Identifier: MIT
"""The configuration of the application QMainWindow"""
import json
import os
from pathlib import Path
from subprocess import CalledProcessError
from typing import Optional

from PySide6.QtCore import Slot
from PySide6.QtGui import QCloseEvent
from PySide6.QtWidgets import QMainWindow, QApplication, QDialog, QMessageBox, QListWidget

import hatch_pyside
import hatch_pyside.gui.app as app
from hatch_pyside import __version__
from hatch_pyside.gui.ui_about import Ui_About
from hatch_pyside.gui.ui_mainwindow import Ui_MainWindow


class MainWindow(QMainWindow, Ui_MainWindow):
    files: list[str]
    project_files: list[str]
    dirty: bool
    json_file: Optional[str] = None

    def __init__(self, projects: list[str], folder: str, name: str):
        """
        The main window for the GUI.

        Args:
            projects:   the list of the configured folders for the plugin
            folder:     the only or selected folder
            name:       the project name
        """
        super().__init__()
        self.setupUi(self)
        self.projects = projects
        self.project = folder
        self.setWindowTitle(f"hatch-pyside - {folder}")
        # do not suggest to change folder when there is only one...
        if len(projects) <= 1:
            self.action_Change_project_folder.setVisible(False)
        self.reload()
        self.proj_name = name

    @Slot()
    def about_qt(self):
        """Displays information about the underlying Qt framework"""
        # noinspection PyUnresolvedReferences
        QApplication.instance().aboutQt()

    @Slot()
    def about(self):
        """Displays the application version"""
        About().exec()

    @Slot()
    def change_project(self):
        """Let the user choose a new project folder"""
        self.project = app.choose_project(self.projects)
        self.setWindowTitle(f"hatch-pyside - {self.project}")
        self.reload()

    @Slot()
    def reload(self):
        """Reload the list of files in the folder and the content of the
         pyproject file
         """
        self.files = [f for f in os.listdir(self.project)
                      if os.path.isfile(os.path.join(self.project, f))]
        for f in self.files:
            if os.path.normcase(f).endswith('.pyproject'):
                with open(Path(self.project) / f) as fd:
                    self.project_files = json.load(fd)['files']
                    self.json_file = f
                    break
        else:
            self.json_file = None
            self.project_files = []
        self.project_files_widget.clear()
        self.files_widget.clear()
        self.project_files_widget.addItems(self.project_files)
        items = [f for f in self.files if f not in self.project_files]
        self.files_widget.addItems(items)
        self.dirty = False

    @Slot()
    def build(self):
        """Builds the dependant files from the Qt source ones"""
        try:
            hatch_pyside.build(self.project)
        except CalledProcessError as e:
            QMessageBox(QMessageBox.Icon.Critical,
                        "Build error",
                        f'Command returned {e.returncode}',
                        informativeText=e.stderr).exec()

        self.reload()

    @Slot()
    def clean(self):
        """Removes the dependant files"""
        try:
            hatch_pyside.clean(self.project)
        except CalledProcessError as e:
            QMessageBox(QMessageBox.Icon.Warning,
                        "Clean error",
                        f'Command returned {e.returncode}',
                        informativeText=e.stderr).exec()
        self.reload()

    @Slot()
    def remove(self):
        """Removes the selected files from the .pyproject file"""
        self.dirty |= move_sel(self.project_files_widget, self.files_widget)

    @Slot()
    def add(self):
        """Adds the selected files to the .pyproject file"""
        self.dirty |= move_sel(self.files_widget, self.project_files_widget)

    @Slot()
    def save(self):
        """Saves the .pyproject file"""
        self.project_files = sorted(
            [self.project_files_widget.item(row).text()
             for row in range(self.project_files_widget.count())])
        json_file = self.json_file
        if json_file is None:
            json_file = f'{self.proj_name}.pyproject'
        with open(Path(self.project) / json_file, 'w') as fd:
            # noinspection PyTypeChecker
            json.dump({'files': self.project_files}, fd)
            self.json_file = json_file
            self.dirty = False

    def closeEvent(self, event: QCloseEvent):
        # If the .pyproject has been change and not yet saved,
        #  suggests to save it before terminating the application session
        if self.dirty:
            buttons = (QMessageBox.StandardButton.Save
                       | QMessageBox.StandardButton.Discard
                       | QMessageBox.StandardButton.Cancel)
            cr = QMessageBox(QMessageBox.Icon.Warning, "Save?",
                           "The project file has unsaved changes"
                           "do you want to quit anyway?",
                           standardButtons=buttons).exec()
            if cr == QMessageBox.StandardButton.Cancel:
                event.ignore()
                return
            elif cr == QMessageBox.StandardButton.Save:
                self.save()
        event.accept()

    @Slot()
    def lupdate(self):
        """Updates the translation files (*.ts)"""


def move_sel(src: QListWidget, dest: QListWidget) -> bool:
    # move the selection from one source QListWidget to a destination one
    sel = src.selectedItems()
    if len(sel) == 0:
        return False
    for item in dest.selectedItems():
        item.setSelected(False)
    for item in sel:
        it = src.takeItem(src.row(item))
        dest.addItem(it)
        it.setSelected(True)
    return True


def clear_selection(list_widget: QListWidget) -> None:
    for item in list_widget.selectedItems():
        item.setSelected(False)


class About(QDialog, Ui_About):
    """A QDialog to display the application version"""
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.version.setText(__version__)
